\ To test some of the ANS Forth Core Ext word set, version 0.1

\ Copyright (C) Gerry Jackson 2006

\ This library is free software; you can redistribute it and/or
\ modify it any way.

\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

\ The tests are not claimed to be comprehensive or correct 

\ --------------------------------------------------------------------
\ Version 0.1   Oct 2006 First version released

\ --------------------------------------------------------------------

\ This is only a partial test of the core extension words.
\ The tests are based on John Hayes test program for the core word set

\ Words tested in this file are:
\     true false :noname ?do value to case of endof endcase
\     c" convert compile, [compile] save-input restore-input

\ --------------------------------------------------------------------
\ Assumptions:
\     - a case insensitive system. Strictly speaking ANS Forth words
\       should be in upper case only, this file is mostly lower case
\     - tester.fr has been loaded prior to this file
\ --------------------------------------------------------------------

Testing Core Extension words

decimal

Testing true false

{ true  -> 0 invert }
{ false -> 0 }

\ --------------------------------------------------------------------

Testing :noname

variable nn1
variable nn2
:noname 1234 ; nn1 !
:noname 9876 ; nn2 !
{ nn1 @ execute -> 1234 }
{ nn2 @ execute -> 9876 }

\ --------------------------------------------------------------------

Testing ?do

: qd ?do i loop ;
{ 789 789 qd -> }
{ -9876 -9876 qd -> }
{ 5 0 qd -> 0 1 2 3 4 }

: qd1 ?do i 10 +loop ;
{ 50 1 qd1 -> 1 11 21 31 41 }

: qd2 ?do i 3 > if leave else i then loop ;
{ 5 -1 qd2 -> -1 0 1 2 3 }

\ --------------------------------------------------------------------

Testing value to

{ 111 value v1 -999 value v2 -> }
{ v1 -> 111 }
{ v2 -> -999 }
{ 222 to v1 -> }
{ v1 -> 222 }
{ : vd1 v1 ; -> }
{ vd1 -> 222 }
{ : vd2 to v2 ; -> }
{ v2 -> -999 }
{ -333 vd2 -> }
{ v2 -> -333 }
{ v1 -> 222 }

\ --------------------------------------------------------------------

Testing case of endof endcase

: cs1 case 1 of 111 endof
           2 of 222 endof
           3 of 333 endof
           >r 999 r>
      endcase
;

{ 1 cs1 -> 111 }
{ 2 cs1 -> 222 }
{ 3 cs1 -> 333 }
{ 4 cs1 -> 999 }

: cs2 >r case -1 of case r@ 1 of 100 endof
                            2 of 200 endof
                           >r -300 r>
                    endcase
                 endof
              -2 of case r@ 1 of -99  endof
                            >r -199 r>
                    endcase
                 endof
                 >r 299 r>
         endcase r> drop
;

{ -1 1 cs2 ->  100 }
{ -1 2 cs2 ->  200 }
{ -1 3 cs2 -> -300 }
{ -2 1 cs2 -> -99  }
{ -2 2 cs2 -> -199 }
{  0 2 cs2 ->  299 }

\ --------------------------------------------------------------------

Testing c" convert

: cq1 c" 1234" ;

{ cq1 count evaluate -> 1234 }

variable conv1
variable conv2
{ : cq2 c" .98765A123:" ; -> }
{ cq2 count drop conv1 ! -> }
{ conv1 @ 6 chars + conv2 ! -> }
{ 0 0 conv1 @ convert ->  98765 0 conv2 @ }
{ 3 0 conv1 @ convert -> 398765 0 conv2 @ }
{ 0 0 conv1 @ convert convert -> 98765123. conv2 @ 4 chars + }
		
\ --------------------------------------------------------------------

Testing compile, [compile]

:noname dup + ; constant dup+
{ : q dup+ compile, ; -> }
{ : as [ q ] ; -> }
{ 123 as -> 246 }

{ : [c1] [compile] dup ; immediate -> }
{ 123 [c1] -> 123 123 }                 \ With default compilation semantics
{ :  [c2] [compile] [c1] ; -> }
{ 234 [c2] -> 234 234 }                 \ With an immediate word
{ : [cif] [compile] if ; immediate -> }
{ : [c3] [cif] 111 else 222 then ; -> } \ With special compilation semantics
{ -1 [c3] -> 111 }
{  0 [c3] -> 222 }

\ --------------------------------------------------------------------
\ Cannot automatically test save-input and restore-input from a console source


Testing save-input and restore-input with a file source

variable siv -1 siv !

: NeverExecuted
	." This should never be executed" abort
;

11111 save-input

siv @

[if]
	0 siv !
	restore-input
	NeverExecuted
[else]

Testing the [else] part is executed
22222

[then]

{ -> 11111 0 22222 }	\ 0 comes from restore-input

Testing save-input and restore-input with a string source

variable si_inc 0 si_inc !

: si1
	si_inc @ >in +!
	15 si_inc !
;

: s$ s" save-input si1 restore-input 12345" ;

{ s$ evaluate si_inc @ -> 0 2345 15 }

Testing nested save-input and restore-input

: read_a_line
	refill 0=
	abort" refill failed"
;

0 si_inc !

2variable 2res -1. 2res 2!

: si2
	read_a_line
	read_a_line
	save-input
	read_a_line
	read_a_line
	s$ evaluate 2res 2!
	restore-input
;

\ WARNING: do not delete or insert lines of text after si2 is called
\ otherwise the next test will fail

si2
33333					\ This line should be ignored
2res 2@ 44444		\ restore-input should return to this line

55555
Testing the nested results
{ -> 0 0 2345 44444 55555 }

\ End of warning

\ --------------------------------------------------------------------

cr .( End of Core Extension word tests) cr


