\ To test the ANS Forth Exception word set and extension words

\ Copyright (C) Gerry Jackson 2006

\ This program is free software; you can redistribute it and/or
\ modify it any way.

\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

\ The tests are not claimed to be comprehensive or correct 

\ --------------------------------------------------------------------
\ Version 0.1   Oct 2006 First version released

\ --------------------------------------------------------------------
\ The tests are based on John Hayes test program for the core word set
\ and requires those files to have been loaded

\ Words tested in this file are:
\     catch throw abort abort"

\ --------------------------------------------------------------------
\ Assumptions and dependencies:
\     - running on a case insensitive system. Strictly speaking ANS 
\       Forth words should be in upper case only, this file is mostly
\       lower case
\     - the forth system under test throws an exception with throw
\       code -13 for a word not found by the text interpreter. The
\       undefined word used is $$qweqweqwert$$,  if this happens to be
\       a valid word in your system change the definition of t7 below
\     - tester.fr has been loaded prior to this file
\ --------------------------------------------------------------------

Testing catch throw

decimal

: t1 9 ;
: c1 1 2 3 ['] t1 catch ;
{ c1 -> 1 2 3 9 0 }			\ No throw executed

: t2 8 0 throw ;
: c2 1 2 ['] t2 catch ;
{ c2 -> 1 2 8 0 }				\ 0 throw does nothing

: t3 7 8 9 99 throw ;
: c3 1 2 ['] t3 catch ;
{ c3 -> 1 2 99 }				\ restores stack to catch depth

: t4 1- dup 0> if recurse else 999 throw -222 then ;
: c4 3 4 5 10 ['] t4 catch -111 ;
{ c4 -> 3 4 5 0 999 -111 }	\ Test return stack unwinding

: t5 2drop 2drop 9999 throw ;
: c5 1 2 3 4 ['] t5 catch				\ Test depth restored correctly
	depth >r drop 2drop 2drop r> ;	\ when stack has been emptied
{ c5 -> 5 }

\ --------------------------------------------------------------------

Testing abort abort"
-1	constant exc_abort
-2 constant exc_abort"
-13 constant exc_undef
: t6 abort ;

\ The 77 in t10 is necessary for the second abort" test as the data stack
\ is restored to a depth of 2 when throw is executed. The 77 ensures the top
\ of stack value is known for the results check

: t10 77 swap abort" This should not be displayed" ;
: c6 catch
	case exc_abort  of 11 endof
	     exc_abort" of 12 endof
		  exc_undef  of 13 endof
	endcase
;

{ 1 2 ' t6 c6  -> 1 2 11 }		\ Test that ABORT is caught
{ 3 0 ' t10 c6 -> 3 77 }		\ ABORT" does nothing
{ 4 5 ' t10 c6 -> 4 77 12 }	\ ABORT" caught, no message

\ --------------------------------------------------------------------

Testing a system generated exception

: t7 s" 333 $$qweqweqwert$$ 334" evaluate 335 ;
: t8 s" 222 t7 223" evaluate 224 ;
: t9 s" 111 112 t8 113" evaluate 114 ;

{ 6 7 ' t9 c6 3 -> 6 7 13 3 }			\ Test unlinking of sources

\ --------------------------------------------------------------------

cr .( End of Exception word tests) cr

