\ To test the ANS File Access word set and extension words

\ Copyright (C) Gerry Jackson 2006

\ This program is free software; you can redistribute it and/or
\ modify it any way.

\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

\ The tests are not claimed to be comprehensive or correct 

\ --------------------------------------------------------------------
\ Version 0.2   30 Oct 2006 updated following GForth tests to remove
\               system dependency on file size, to allow for file
\               buffering and to allow for PAD moving around
\         0.1   Oct 2006 First version released

\ --------------------------------------------------------------------
\ The tests are based on John Hayes test program for the core word set
\ and requires those files to have been loaded

\ Words tested in this file are:
\     ( bin close-file create-file delete-file file-position file-size
\     open-file r/o r/w read-file read-line reposition-file resize-file 
\     s" source-id w/o write-file write-line 
\     file-status flush-file rename-file 

\ Words not tested:
\     refill included include-file (as these will likely have been
\     tested in the execution of the test files)
\ --------------------------------------------------------------------
\ Assumptions, dependencies and notes:
\     - running on a case insensitive system. Strictly speaking ANS 
\       Forth words should be in upper case only, this file is mostly
\       lower case
\     - tester.fr has been loaded prior to this file
\     - These tests create files in the current directory, if all goes
\       well these will be deleted. If something fails they may not be
\       deleted. If this is a problem ensure you set a suitable 
\       directory before running this test. There is no ANS standard
\       way of doing this. Also beware the file names used below which
\       are:  fatest1.txt, fatest2.txt and fatest3.txt
\ --------------------------------------------------------------------

Testing File Access word set

decimal

\ --------------------------------------------------------------------

Testing create-file close-file

: fn1 s" test\fatest1.txt" ;
variable fid1

{ fn1 r/w create-file swap fid1 ! -> 0 }
{ fid1 @ close-file -> 0 }

\ --------------------------------------------------------------------

Testing open-file w/o write-line

: line1 s" Line 1" ;

{ fn1 w/o open-file swap fid1 ! -> 0 }
{ line1 fid1 @ write-line -> 0 }
{ fid1 @ close-file -> 0 }

\ --------------------------------------------------------------------

Testing r/o file-position (simple)  read-line 

200 constant bsize
create buf bsize allot
variable #chars

{ fn1 r/o open-file swap fid1 ! -> 0 }
{ fid1 @ file-position -> 0. 0 }
{ buf 100 fid1 @ read-line rot dup #chars ! -> true 0 line1 nip }
{ buf #chars @ line1 compare -> 0 }
{ fid1 @ close-file -> 0 }

\ --------------------------------------------------------------------

Testing r/w write-file reposition-file read-file file-position s"

: line2 s" Line 2 blah blah blah" ;
: rl1 buf 100 fid1 @ read-line ;
2variable fp

{ fn1 r/w open-file swap fid1 ! -> 0 }
{ fid1 @ file-size drop fid1 @ reposition-file -> 0 }
{ fid1 @ file-size -> fid1 @ file-position }
{ line2 fid1 @ write-file -> 0 }
{ 10. fid1 @ reposition-file -> 0 }
{ fid1 @ file-position -> 10. 0 }
{ 0. fid1 @ reposition-file -> 0 }
{ rl1 -> line1 nip true 0 }
{ rl1 rot dup #chars ! -> true 0 line2 nip }
{ buf #chars @ line2 compare -> 0 }
{ rl1 -> 0 false 0 }
{ fid1 @ file-position rot rot fp 2! -> 0 }
{ fp 2@ fid1 @ file-size drop d= -> true }
{ s" " fid1 @ write-line -> 0 }
{ s" " fid1 @ write-line -> 0 }
{ fp 2@ fid1 @ reposition-file -> 0 }
{ rl1 -> 0 true 0 }
{ rl1 -> 0 true 0 }
{ rl1 -> 0 false 0 }
{ fid1 @ close-file -> 0 }

\ --------------------------------------------------------------------

Testing bin read-file file-size

: cbuf buf bsize 0 fill ;
: fn2 s" test\fatest2.txt" ;
variable fid2
: setpad pad 50 0 do i over c! char+ loop drop ;

setpad   \ If anything else is defined setpad must be called again
         \ as pad may move

{ fn2 r/w bin create-file swap fid2 ! -> 0 }
{ pad 50 fid2 @ write-file fid2 @ flush-file -> 0 0 }
{ fid2 @ file-size -> 50. 0 }
{ 0. fid2 @ reposition-file -> 0 }
{ cbuf buf 29 fid2 @ read-file -> 29 0 }
{ pad 29 buf 29 compare -> 0 }
{ pad 30 buf 30 compare -> 1 }
{ cbuf buf 29 fid2 @ read-file -> 21 0 }
{ pad 29 + 21 buf 21 compare -> 0 }
{ fid2 @ file-size drop fid2 @ file-position drop d= -> true }
{ buf 10 fid2 @ read-file -> 0 0 }
{ fid2 @ close-file -> 0 }

\ --------------------------------------------------------------------

Testing resize-file

{ fn2 r/w bin open-file swap fid2 ! -> 0 }
{ 37. fid2 @ resize-file -> 0 }
{ cbuf buf 100 fid2 @ read-file -> 37 0 }
{ pad 37 buf 37 compare -> 0 }
{ pad 38 buf 38 compare -> 1 }
{ 500. fid2 @ resize-file -> 0 }
{ fid2 @ file-size -> 500. 0 }
{ fid2 @ close-file -> 0 }

\ --------------------------------------------------------------------

Testing delete-file

{ fn2 delete-file -> 0 }
{ fn2 r/w bin open-file nip 0= -> false }
{ fn2 delete-file 0= -> false }

\ --------------------------------------------------------------------

Testing multi-line ( comments

{ ( 1 2 3
4 5 6
7 8 9 ) 11 22 33 -> 11 22 33 }

\ --------------------------------------------------------------------

Testing source-id (can only test it does not return 0 or -1)

{ source-id dup -1 = swap 0= or -> false }

\ --------------------------------------------------------------------

Testing rename-file file-status flush-file

: fn3 s" test\fatest3.txt" ;
: >end fid1 @ file-size drop fid1 @ reposition-file ;


{ fn3 delete-file drop -> }
{ fn1 fn3 rename-file 0= -> true }
{ fn1 file-status nip 0= -> false }
{ fn3 file-status nip 0= -> true }  \ Can't test an undefined return value
{ fn3 r/w open-file swap fid1 ! -> 0 }
{ >end -> 0 }
{ s" Final line" fid1 @ write-line -> 0 }
{ fid1 @ flush-file -> 0 }		\ Can only test flush-file doesn't fail
{ fid1 @ close-file -> 0 }

\ Tidy the test folder
{ fn3 delete-file drop -> }

\ --------------------------------------------------------------------

cr .( End of File-Access word tests) cr
