\ To test some of the ANS Forth Programming Tools and extension wordset

\ Copyright (C) Gerry Jackson 2006

\ This program is free software; you can redistribute it and/or
\ modify it any way.

\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

\ The tests are not claimed to be comprehensive or correct 

\ --------------------------------------------------------------------
\ Version 0.2   30 Oct 2006 updated following GForth test to avoid
\               changing stack depth during a colon definition
\         0.1   Oct 2006 First version released

\ --------------------------------------------------------------------
\ The tests are based on John Hayes test program for the core word set
\ and requires those files to have been loaded

\ Words tested in this file are:
\     ahead [if] [else] [then] cs-pick cs-roll
\     

\ Words not tested:
\     .s ? dump see words
\     ;code assembler bye code editor forget state 
\ --------------------------------------------------------------------
\ Assumptions and dependencies:
\     - running on a case insensitive system. Strictly speaking ANS 
\       Forth words should be in upper case only, this file is mostly
\       lower case
\     - tester.fr has been loaded prior to this file
\ --------------------------------------------------------------------

Testing ahead

{ : pt1 ahead 1111 2222 then 3333 ; -> }
{ pt1 -> 3333 }

\ --------------------------------------------------------------------

Testing [if] [else] [then]

{ true  [if] 111 [else] 222 [then] -> 111 }
{ false [if] 111 [else] 222 [then] -> 222 }

{ true  [if] 1
				 2
		  [ELSE]
				   3
					4
		  [THEN]		-> 1 2 }
{ false [IF]
				   1 2
		  [else]
					3 4 [then] -> 3 4 }

{ true  [if] 1 true  [if] 2 [else] 3 [then] [else] 4 [then] -> 1 2 }
{ false [if] 1 true  [if] 2 [else] 3 [then] [else] 4 [then] -> 4 }
{ true  [if] 1 false [if] 2 [else] 3 [then] [else] 4 [then] -> 1 3 }
{ false [if] 1 false [if] 2 [else] 3 [then] [else] 4 [then] -> 4 }

\ --------------------------------------------------------------------

Testing immediacy of [if] [else] [then]

{ : pt2 [  0 ] [if] 1111 [else] 2222 [then]  ; pt2 -> 2222 }
{ : pt3 [ -1 ] [if] 3333 [else] 4444 [endif] ; pt3 -> 3333 }

\ --------------------------------------------------------------------

Testing cs-pick and cs-roll

\ Test pt4 based on example in ANS document p 176.

: ?repeat
	0 cs-pick postpone until
; immediate

variable pt4

{ : pt5	( n1 -- )
		pt4 !
		begin
			-1 pt4 +!
			pt4 @ 4 <= ?repeat \ Back to begin if false
			111
			pt4 @ 3 <= ?repeat
			222
			pt4 @ 2 <= ?repeat
			333
			pt4 @ 1 =
		until
; -> }

{ 6 pt5 -> 111 111 222 111 222 333 111 222 333 }


{ : ?done postpone if 1 cs-roll ; immediate -> }  \ Synonym for while
{ : pt6 
		>r
		begin
			r@
		?done
			r@
			r> 1- >r
		repeat
		r> drop
	; -> }

{ 5 pt6 -> 5 4 3 2 1 }

: mix_up 2 cs-roll ; immediate  \ cs-rot

: pt7		( f3 f2 f1 -- ? )
	if 1111 -rot				( -- 1111 f3 f2 )     ( cs: -- orig1 )
		if 2222 swap			( -- 1111 2222 f3 )   ( cs: -- orig1 orig2 )
			if												    ( cs: -- orig1 orig2 orig3 )
				3333 mix_up		( -- 1111 2222 3333 ) ( cs: -- orig2 orig3 orig1 )
			endif												 ( cs: -- orig2 orig3 )
			4444			\ Hence failure of first if comes here and falls through
		endif													 ( cs: -- orig2 )
		5555				\ Failure of 3rd if comes here
	endif														 ( cs: -- )
	6666					\ Failure of 2nd if comes here
;

{ -1 -1 -1 pt7 -> 1111 2222 3333 4444 5555 6666 }
{  0 -1 -1 pt7 -> 1111 2222 5555 6666 }
{  0  0 -1 pt7 -> 1111 0    6666 }
{  0  0  0 pt7 -> 0    0    4444 5555 6666 }

: [1cs-roll] 1 cs-roll ; immediate

{ : pt8 
		>r
		ahead 111
		begin 222 
			[1cs-roll]
			then
			333
			r> 1- >r
			r@ 0<
		until
		r> drop
	; -> }

{ 1 pt8 -> 333 222 333 }

cr .( End of Programming Tools word tests) cr
